package TKL;

use XML::LibXML;
use TKL::File;
use Carp;
use strict;
use vars qw($xml_header $tkl_suffix $schema_suffix $uri_element_type $schema_NS_URI);


$xml_header = "<?xml version=\"1.0\" encoding=\"ISO-8859-1\" standalone=\"yes\"?>";
$tkl_suffix = "tkl";
$schema_suffix = "xsd";
$schema_NS_URI = "http://www.w3.org/2001/XMLSchema";
$uri_element_type = "xs:anyURI";

sub new {
    my ($class, @rest) = @_;
    my %args = (tkl_config	=> "tkl.config",
    		oai_dir		=> "oai",
		schema_dir	=> "schemas",
		@rest);
    my $self = \%args;

    bless $self, $class;
    $self->find_portal_root unless defined($self->{root});
    $self->{libxml} = new XML::LibXML;

    return $self;
}


sub dom {
    return shift->{libxml};
}


sub document_element {
    my ($self, $xml_file) = @_;
    my $doc = $self->dom->parse_file($xml_file);

    return $doc->documentElement;
}
    

sub find_portal_root {
    my $self = shift;
    my $pwd;

    return $self->{root} if defined($self->{root});
    chomp($pwd = `pwd`);
    my @path = split /\//, $pwd;
    while (@path) {
	my $super_dir = join "/", @path;
	my $file = "$super_dir/" . $self->{tkl_config};
	if (-f $file) {
	    return $self->{root} = $super_dir;
	}
	pop(@path);
    }
    croak "$0: Unable to find portal root";
}
    

sub xml_trim {
    my ($self, $cdata) = @_;

    $cdata =~ s/^\s+//g;
    $cdata =~ s/\s+$//g;
    $cdata =~ s/&amp;/&/g;
    $cdata =~ s/&/&amp;/g;
    $cdata =~ s/</&lt;/g;
    $cdata =~ s/>/&gt;/g;

    return $cdata;
}


sub dom2hash {
    my ($self, $dom, $href) = @_;
    my $name = $dom->nodeName();
    my $cdata = "";
    my $offset;
    
    if (ref($href->{$name}) eq 'ARRAY') {
	$offset = $#{$href->{$name}} + 1;
    } else {
	$href->{$name} = [];
	$offset = 0;
    }
    foreach my $kid ($dom->childNodes()) {
	my $type = $kid->getType();
	if ($type == XML_ELEMENT_NODE) {
	    my $subtree = $href->{$name}->[$offset]->{nodeset};
	    if (!defined($subtree)) {
		$subtree = $href->{$name}->[$offset]->{nodeset} = {};
	    }
	    $self->dom2hash($kid, $subtree);
	} elsif ($type == XML_TEXT_NODE) {
	    $cdata .= $kid->nodeValue();
	} else {
	    warn "$0 - dom2hash: Uknown node type '$type'";
	}
    }
    $href->{$name}->[$offset]->{content} = $self->xml_trim($cdata);
    if (my @attr = $dom->attributes) {
	foreach my $entry (@attr) {
	    $href->{$name}->[$offset]->{attr}->{$entry->nodeName()} = $entry->nodeValue();
	}
    }
}


sub parse {
    my ($self, $xml_file) = @_;
    my $root = $self->document_element($xml_file);

    $self->dom2hash($root, my $href = {});

    return $href;
}


sub browse {
    my ($self, $dir) = @_;
    my $files = [];
    my $dirs = [];
    my $root = $self->{root};
    local *DIR;

    $dir = "" unless defined($dir);
    $dir = "$root/$dir/";
    opendir DIR, $dir or die "$0 - browse: Unable to open directory '$dir': $!";
    while (defined(my $entry = readdir DIR)) {
	next if $entry =~ /^\.\.?/;
	my $file = "$dir/$entry";
	if (-d $file) {
	    push @$dirs, $entry;
	} elsif ($entry =~ /\.$tkl_suffix$/) {
	    $file =~ s/\/+/\//g;
	    push @$files, new TKL::File(file => $file);
	}
    }
    closedir DIR or warn "$0 - browse: Unable to close directory '$dir' properly: $!";
    
    return {files => $files, dirs => $dirs};
}

1;

## $Log: TKL.pm,v $
## Revision 1.5  2003/06/11 11:49:48  sondberg
## Multiple slashes are replaced by a single slash.
##
## Revision 1.4  2002/11/28 13:07:54  sondberg
## Global replacements of invalid entities.
##
## Revision 1.3  2002/11/28 13:00:15  sondberg
## Correct handling of entities.
##
## Revision 1.2  2002/11/27 15:00:17  sondberg
## Corrected a bug in find_portal_root.
##
## Revision 1.1  2002/11/27 14:52:31  sondberg
## Added this file to repository.
##
